# Percorso del file di output
$outputFilePath = "output.txt"

# Verifica se il file di output esiste ed è in sola lettura
$outputFileInfo = Get-Item -Path $outputFilePath -ErrorAction SilentlyContinue
if (($outputFileInfo -ne $null) -and ($outputFileInfo.Attributes -band [System.IO.FileAttributes]::ReadOnly)) {
    Write-Host "ERRORE: File di output in sola lettura. Probabilmente sorteggio precedentemente effettuato." -ForegroundColor DarkRed
    return
}

# Ottieni l'ora e la data correnti
$currentDateTime = Get-Date

# Salva la data e l'ora nel formato desiderato
$dateTimeString = $currentDateTime.ToString("dd/MM/yyyy HH:mm:ss")

# mostra e scrivi nel file di output la data e l'ora di esecuzione
Write-Host "Data e ora di esecuzione dello script: $dateTimeString" -ForegroundColor Yellow
Add-Content -Path $outputFilePath -Value "Data e ora di esecuzione dello script: $dateTimeString" -Encoding UTF8

# Inizializza l'algoritmo di hash
$hashAlgorithm = [System.Security.Cryptography.HashAlgorithm]::Create("SHA256")

# Calcola l'hash dello script PowerShell in esecuzione
$scriptPath = $MyInvocation.MyCommand.Path
$scriptContent = Get-Content -Path $scriptPath -Encoding UTF8 | Out-String
$scriptHash = $hashAlgorithm.ComputeHash([System.Text.Encoding]::UTF8.GetBytes($scriptContent))
$scriptHashString = [System.BitConverter]::ToString($scriptHash).Replace("-", "")

# Scrivi l'hash dello script nel file di output
Write-Host "Hash dello script PowerShell: $scriptHashString" -ForegroundColor Yellow
Add-Content -Path $outputFilePath -Value "Hash dello script PowerShell: $scriptHashString" -Encoding UTF8

# Carica il contenuto del file XML utilizzando l'encoding UTF-8
$xmlPath = "xml_sorteggio.xml"
$xmlContent = Get-Content -Path $xmlPath -Encoding UTF8 | Out-String

# Calcola l'hash del file XML
$xmlHash = $hashAlgorithm.ComputeHash([System.Text.Encoding]::UTF8.GetBytes($xmlContent))
$xmlHashString = [System.BitConverter]::ToString($xmlHash).Replace("-", "")

# Visualizza Hash iniziale del file XML, utilizzato nella Hash Chain
Write-Host "Hash del file XML: $xmlHashString" -ForegroundColor Yellow

# Scrivi Hash del file XML, utilizzato nella Hash Chain nel file di output
Add-Content -Path $outputFilePath -Value "Hash del file XML: $xmlHashString" -Encoding UTF8

# Calcola l'hash iniziale mediante concatenando date e ora, hash dello script ed hash del file xml
$initialHash = $hashAlgorithm.ComputeHash([System.Text.Encoding]::UTF8.GetBytes("$dateTimeString$scriptHashString$xmlHashString"))
$initialHashString = [System.BitConverter]::ToString($initialHash).Replace("-", "")

# Visualizza Hash iniziale
Write-Host "Hash iniziale: $initialHashString" -ForegroundColor Yellow
Write-Host 
Write-Host

# Scrivi Hash iniziale, utilizzato nella Hash Chain nel file di output
Add-Content -Path $outputFilePath -Value "Hash iniziale: $initialHashString" -Encoding UTF8
Add-Content -Path $outputFilePath -Value "" -Encoding UTF8
Add-Content -Path $outputFilePath -Value "" -Encoding UTF8

# Leggi il contenuto del file XML come oggetto XML
$xml = [xml]$xmlContent

# Estrai le commissioni dal file XML
$commissioni = $xml.commissioni.commissione


# Inizializza il generatore di numeri casuali crittograficamente sicuri
$randomGenerator = [System.Security.Cryptography.RandomNumberGenerator]::Create()

# Inizializza l'hash precedente con l'hash iniziale del file XML
$previousHash = $initialHashString

# Ciclo per ogni commissione nel file XML
foreach ($comm in $commissioni) {
    # Il blocco try viene eseguito sequenzialmente, se il flusso del programma viene interrotto passa al blocco catch
    try {
        # Estrai i dati del designato dalla commissione
        $designatoNominativo = $comm.designato.designato_nominativo
        $designatoGenere = $comm.designato.designato_genere
        $designatoUniv = $comm.designato.designato_univ
		
        # In caso di sorteggio supplettivo, estrai i dati del nominato
        if ($comm.nominato) {
            $nominatoNominativo = $comm.nominato.nominato_nominativo
            $nominatoGenere = $comm.nominato.nominato_genere
            $nominatoUniv = $comm.nominato.nominato_univ
        }

        # Viene creato un array chiamato $sorteggiati per memorizzare i componenti sorteggiabili
        $sorteggiabili = $comm.sorteggiabile

        # Filtra i sorteggiabili in base al genere del designato
        $sorteggiabiliGenereOpposto = $sorteggiabili | Where-Object { $_.sorteggiabile_genere -ne $designatoGenere }

        # Resetta la Commissione per la nuova commissione
        $commissione = @()

        # Viene creato un array di byte $randomBytes con una lunghezza di 4 byte. Questo array conterrà i byte generati casualmente 
        $randomBytes = New-Object byte[] 4
        
        # In caso di sorteggio supplettivo, non si procede a sorteggio, ed $sorteggaito viene valorizzato con il nominato, sorteggaito in un precedente sorteggio
		
        # Inizializza $sorteggiato come oggetto separato in caso di sorteggio suplettivo
        if ($comm.nominato) {
            $sorteggiato = [pscustomobject]@{
                sorteggiabile_nominativo = $comm.nominato.nominato_nominativo
                sorteggiabile_genere = $comm.nominato.nominato_genere
                sorteggiabile_univ = $comm.nominato.nominato_univ
            }
        }
        else {
            <# Si procede al primo sorteggio.
            Il metodo GetBytes() è utilizzato per ottenere una sequenza di byte casuali per riempire l'array $randomBytes.
            Questo metodo fa parte della classe System.Security.Cryptography.RandomNumberGenerator, la quale offre un'implementazione di un generatore di numeri casuali 
            che garantisce l'alta qualità e la sicurezza crittografica dei numeri generati.
            I byte casuali dell'array $randomBytes vengono infine convertiti in un numero intero utilizzando il metodo ToInt32() della classe BitConverter, 
            producendo un numero intero casuale crittograficamente sicuro. Dopo il sorteggio, viene rimosso il sorteggiato dagli array dei sorteggiabili.#>
            $randomGenerator.GetBytes($randomBytes)
            $randomNumber = [BitConverter]::ToInt32($randomBytes, 0)
            $sorteggiato = $sorteggiabili[$randomNumber % $sorteggiabili.Count]
            $sorteggiabili = $sorteggiabili | Where-Object { $_ -ne $sorteggiato }
        }

        # Si procede con il secondo sorteggio
        
        # Genera un altro numero casuale crittograficamente sicuro, analogamente a quanto commentato precedentemente 
        $randomGenerator.GetBytes($randomBytes)
        $randomNumber = [BitConverter]::ToInt32($randomBytes, 0)
        
        if ($designatoGenere -eq $sorteggiato.sorteggiabile_genere -and $sorteggiabiliGenereOpposto.Count -gt 1) {
            # Se Il genere del componente designato e quello del primo sorteggiato coincidono e ci sono almeno due sorteggiabili di genere opposto, viene garantita la rappresentanza di genere con l'ultimo sorteggio

            # Sorteggia casualmente il secondo componente tra quelli di genere opposto, analogamente a quanto commentato precedentemente
            $sorteggiato2 = $sorteggiabiliGenereOpposto[$randomNumber % $sorteggiabiliGenereOpposto.Count]
			

        } else {
            # negli altri casi si procede con un secondo sorteggio tra tutti i sorteggiabili
                                    
            # Sorteggia casualmente il secondo componente, analogamente a quanto commentato precedentemente        
            $sorteggiato2 = $sorteggiabili[$randomNumber % $sorteggiabili.Count]
        } 

        # Aggiungi i componenti della Commissione
        $commissione += "$($designatoNominativo) - $($designatoGenere) - $($designatoUniv)"
        $commissione += "$($sorteggiato.sorteggiabile_nominativo) - $($sorteggiato.sorteggiabile_genere) - $($sorteggiato.sorteggiabile_univ)"
        $commissione += "$($sorteggiato2.sorteggiabile_nominativo) - $($sorteggiato2.sorteggiabile_genere) - $($sorteggiato2.sorteggiabile_univ)"
        
        # Memorizza la composizione della Commissione su tre righe
        $commissioneCompleta = $commissione -join "`n"
        
        # Calcola l'hash corrente mediante la composizione della commissione concatenata con l'hash precedente
        $currentHash = $hashAlgorithm.ComputeHash([System.Text.Encoding]::UTF8.GetBytes("$commissioneCompleta$previousHash"))
        $currentHashString = [System.BitConverter]::ToString($currentHash).Replace("-", "")

        # Aggiorna l'hash precedente con l'hash appena calcolato
        $previousHash = $currentHashString

        # Scrivi i risultati del sorteggio nel file di output
        Add-Content -Path $outputFilePath -Value "Codice Procedura: $($comm.codice_procedura)" -Encoding UTF8
        Add-Content -Path $outputFilePath -Value "Componente designato: $designatoNominativo - $designatoGenere - $designatoUniv" -Encoding UTF8
		if ($comm.nominato) { Add-Content -Path $outputFilePath -Value "Componente precedentemente nominato: $($sorteggiato.sorteggiabile_nominativo) - $($sorteggiato.sorteggiabile_genere) - $($sorteggiato.sorteggiabile_univ)" -Encoding UTF8 }
        Add-Content -Path $outputFilePath -Value "" -Encoding UTF8
        Add-Content -Path $outputFilePath -Value "Composizione della Commissione a seguito del sorteggio:" -Encoding UTF8
        Add-Content -Path $outputFilePath -Value "$commissioneCompleta" -Encoding UTF8
        Add-Content -Path $outputFilePath -Value "" -Encoding UTF8
        Add-Content -Path $outputFilePath -Value "Hash corrente Commissione $($comm.codice_procedura): $currentHashString" -Encoding UTF8 
        Add-Content -Path $outputFilePath -Value "" -Encoding UTF8

        # Stampa i risultati del sorteggio a terminale
        Write-Host "Codice Procedura: $($comm.codice_procedura)" -ForegroundColor DarkRed
        Write-Host "Componente designato: $designatoNominativo - $designatoGenere - $designatoUniv"
        if ($comm.nominato) { Write-Host "Componente precedentemente nominato: $($sorteggiato.sorteggiabile_nominativo) - $($sorteggiato.sorteggiabile_genere) - $($sorteggiato.sorteggiabile_univ)" }
		Write-Host
        Write-Host "Composizione della Commissione a seguito del sorteggio:" -ForegroundColor DarkBlue
        Write-Host "$commissioneCompleta"
        Write-Host
        Write-Host "Hash corrente Commissione $($comm.codice_procedura): $currentHashString" -ForegroundColor Yellow
        Write-Host
        Write-Host
    } catch {
        Write-Host "Si è verificato un errore durante l'esecuzione del programma:" -ForegroundColor Red
        Write-Host $_.Exception.Message
    }
}

# Chiudi il generatore di numeri casuali crittograficamente sicuri
$randomGenerator.Dispose()


# Imposta l'attributo "ReadOnly" del file di output
Set-ItemProperty -Path $outputFilePath -Name IsReadOnly -Value $true

Write-Host "Sorteggio completato. I risultati sono stati scritti nel file di output: $outputFilePath" -ForegroundColor Green
Write-Host

# SIG # Begin signature block
# MIIJdwYJKoZIhvcNAQcCoIIJaDCCCWQCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCBeyuNhbeIuObSK
# nexZOWdzcJ2BETafvFQ5y8rDL19466CCBqYwggaiMIIFiqADAgECAhMUAAA4ACOW
# WHrkkDSGAAIAADgAMA0GCSqGSIb3DQEBDAUAMF4xEjAQBgoJkiaJk/IsZAEZFgJp
# dDEWMBQGCgmSJomT8ixkARkWBnBvbGltaTEZMBcGCgmSJomT8ixkARkWCXJlc291
# cmNlczEVMBMGA1UEAxMMUkVTUE9MSU1JLUNBMB4XDTIzMDcwNzA3MjcyOFoXDTI1
# MDcwNjA3MjcyOFowgYQxEjAQBgoJkiaJk/IsZAEZFgJpdDEWMBQGCgmSJomT8ixk
# ARkWBnBvbGltaTEZMBcGCgmSJomT8ixkARkWCXJlc291cmNlczEOMAwGA1UECxMF
# QVNJQ1QxFzAVBgNVBAsTDlNlcnZpY2VBY2NvdW50MRIwEAYDVQQDEwlzaWduIGNv
# ZGUwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQDiqhs4W/hIh8Aetq/1
# vpmFo/zsr3sW99pTbAfFfx9W3/TLME4qqHqq0PY1fMIera2RpaH+qVzR0D94HsUh
# 3XqnFt093o2eeNwwS00+AixM+vyaAvsM4nKq9bOGiaMieD2XN5KGVUp4qo1EP014
# lknMaDNqkewO8ip+2o5UJKav1VqY0/vUEANpqclUsrnzA/6Vo+uDEK5baejST1m8
# KVmEs7fX+JwZunr579CnjRRzd/VyUk+CGBga1AIINGRiN3wm9hMit2/vvlc2upvo
# qpUFEWGlFAu1SibRWUc0CJCXymS3pJMV3SBHbXSoTFIxt9UNugw3joWBtK2NVy8O
# eq0BAgMBAAGjggMwMIIDLDA8BgkrBgEEAYI3FQcELzAtBiUrBgEEAYI3FQitzguC
# mLINh6mfC4S3sQ2EsPFyfYKD9zWGrtU3AgFkAgEEMBMGA1UdJQQMMAoGCCsGAQUF
# BwMDMA4GA1UdDwEB/wQEAwIHgDAbBgkrBgEEAYI3FQoEDjAMMAoGCCsGAQUFBwMD
# MB0GA1UdDgQWBBShFtxtLTEqeK0E7GXgtp3orXUaXDAfBgNVHSMEGDAWgBTT9fh4
# E9CpU/OjasF4LdDRws9HuzCB2QYDVR0fBIHRMIHOMIHLoIHIoIHFhoHCbGRhcDov
# Ly9DTj1SRVNQT0xJTUktQ0EsQ049VldSRVNMSUNBMDEsQ049Q0RQLENOPVB1Ymxp
# YyUyMEtleSUyMFNlcnZpY2VzLENOPVNlcnZpY2VzLENOPUNvbmZpZ3VyYXRpb24s
# REM9cmVzb3VyY2VzLERDPXBvbGltaSxEQz1pdD9jZXJ0aWZpY2F0ZVJldm9jYXRp
# b25MaXN0P2Jhc2U/b2JqZWN0Q2xhc3M9Y1JMRGlzdHJpYnV0aW9uUG9pbnQwggEC
# BggrBgEFBQcBAQSB9TCB8jCBtgYIKwYBBQUHMAKGgalsZGFwOi8vL0NOPVJFU1BP
# TElNSS1DQSxDTj1BSUEsQ049UHVibGljJTIwS2V5JTIwU2VydmljZXMsQ049U2Vy
# dmljZXMsQ049Q29uZmlndXJhdGlvbixEQz1yZXNvdXJjZXMsREM9cG9saW1pLERD
# PWl0P2NBQ2VydGlmaWNhdGU/YmFzZT9vYmplY3RDbGFzcz1jZXJ0aWZpY2F0aW9u
# QXV0aG9yaXR5MDcGCCsGAQUFBzABhitodHRwOi8vVldSRVNMSUNBMDEucmVzb3Vy
# Y2VzLnBvbGltaS5pdC9vY3NwMDgGA1UdEQQxMC+gLQYKKwYBBAGCNxQCA6AfDB1z
# aWduLmNvZGVAcmVzb3VyY2VzLnBvbGltaS5pdDBOBgkrBgEEAYI3GQIEQTA/oD0G
# CisGAQQBgjcZAgGgLwQtUy0xLTUtMjEtMTM0NTMyNTU4MC04ODU5MDMyMC0zMTM2
# MjU0NTQ5LTEyODkwMA0GCSqGSIb3DQEBDAUAA4IBAQA5MAdrgNAAJqugGSkK7yyA
# FAAFzWlS443m+qtQFnN8mjatdCDm26xN8/srqBzwqKCNenkhJjw8ne173ppgO9Um
# /5yaWimSlsyLswrzuk9UxDjIeAh+gL3HKamZB53/kmeKpKLOkGR9sbUMV/mYKKUf
# AIRHsnMQQODcEBv2022E+Fg9MCw8LOMT+p6uB2zaXdubvkEtI1W4KtV4USHnC1zv
# U86p5eoe+FhqphaDgoK9jW7eP+dbSxM68WVe8EJ1RHyyeEXdBNoHs0ycJ0/oKYX/
# ej1ZXOJuVJuEtUlIxRBrPIBGfaOQ7rT2iIIL45tJ7sAXmK09lvpU6SIlI1ehylIp
# MYICJzCCAiMCAQEwdTBeMRIwEAYKCZImiZPyLGQBGRYCaXQxFjAUBgoJkiaJk/Is
# ZAEZFgZwb2xpbWkxGTAXBgoJkiaJk/IsZAEZFglyZXNvdXJjZXMxFTATBgNVBAMT
# DFJFU1BPTElNSS1DQQITFAAAOAAjllh65JA0hgACAAA4ADANBglghkgBZQMEAgEF
# AKCBhDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgor
# BgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMC8GCSqGSIb3
# DQEJBDEiBCAEpbCohgcJDyyd7jfh7P5FEJE4xL3q6hHdUD8pAnA+/jANBgkqhkiG
# 9w0BAQEFAASCAQAIw7deHa5J+GpUZukQ3FF0FLLpBKES606vjdhdS5DPvVdhx63G
# g2cGIZa//MlIfwiDa+rYs9UZszZBMM9hrbzIrVHB7VMIN8oE8ULCkv2hynJi6A+D
# UtWb5VEOUBn3n0s0ScXNvKtfZ20Q+Bysa6IXADkoEigHQYUiSWS3aS0KF0h2P3AI
# oEbue1ioVfuuNUTSy6c4PRCYhbbuEsDZDUAoDGKsZCljq21njjevLs3QvrhpiyBa
# KdOMMlsUXvK8y1D0zda+ZzYgzzOX2d/CURgz/K5LHxzT+Sl151o2VP3eyabwVbv9
# Xh06riAQ+bU/X7Wrab/4oIPkHoKZBc8hWACn
# SIG # End signature block
