[Console]::OutputEncoding = [System.Text.Encoding]::UTF8

# Percorso dello script di sorteggio
$scriptPath = "sorteggio.ps1"

# Carica il contenuto dello script di sorteggio  utilizzando l'encoding UTF-8
$scriptContent = Get-Content -Path $scriptPath -Encoding UTF8 | Out-String

# Percorso del file XML utilizzato nello script di sorteggio
$xmlPath = "xml_sorteggio.xml"

# Carica il contenuto del file XML utilizzando l'encoding UTF-8
$xmlContent = Get-Content -Path $xmlPath -Encoding UTF8 | Out-String

# Inizializza l'algoritmo di hash
$hashAlgorithm = [System.Security.Cryptography.HashAlgorithm]::Create("SHA256")

# Calcola l'hash dello script di sorteggio
$scriptHash = $hashAlgorithm.ComputeHash([System.Text.Encoding]::UTF8.GetBytes($scriptContent))
$scriptHashString = [System.BitConverter]::ToString($scriptHash).Replace("-", "")

# Calcola l'hash del file XML
$xmlHash = $hashAlgorithm.ComputeHash([System.Text.Encoding]::UTF8.GetBytes($xmlContent))
$xmlHashString = [System.BitConverter]::ToString($xmlHash).Replace("-", "")

# Percorso del file di output contenente i risultati dei sorteggi
$outputFilePath = "output.txt"

# Carica il contenuto del file di output
$outputContent = Get-Content -Path $outputFilePath -Raw
$outputContent = $outputContent.Trim()  # Rimuovi gli spazi bianchi iniziali e finali

# Estrai data e ora di esecuzione del sorteggio
$dateHashStringFile = ($outputContent | Select-String -Pattern "Data e ora di esecuzione dello script: (.+)" | ForEach-Object { $_.Matches.Groups[1].Value }).Trim()

# Estrai l'hash dello script
$scriptHashStringFile = ($outputContent | Select-String -Pattern "Hash dello script PowerShell: (.+)" | ForEach-Object { $_.Matches.Groups[1].Value }).Trim()

# Estrai l'hash del file xml
$xmlHashStringFile = ($outputContent | Select-String -Pattern "Hash del file XML: (.+)" | ForEach-Object { $_.Matches.Groups[1].Value }).Trim()

# Estrai l'hash iniziale
$initialHashStringFile = ($outputContent | Select-String -Pattern "Hash iniziale: (.+)" | ForEach-Object { $_.Matches.Groups[1].Value }).Trim()

# Calcola l'hash iniziale sulla base della data e ora salvate in output, l' hash dello script e l hash del file xml calcolati dai file stessi
$initialHash = $hashAlgorithm.ComputeHash([System.Text.Encoding]::UTF8.GetBytes("$dateHashStringFile$scriptHashString$xmlHashString"))
$initialHashString = [System.BitConverter]::ToString($initialHash).Replace("-", "")

# Verifica che l'hash dello script calcolato e quello nel file output coincidano
if ($scriptHashString -ne $scriptHashStringFile) {
    Write-Host "ERRORE: hash chain compromessa! Hash dello script di sorteggio diverso da quello indicato nel file output." -ForegroundColor Red
    Write-Host
    Write-Host "Hash dello script nel file di output: $scriptHashStringFile"
    Write-Host "Hash atteso: $scriptHashString"
    Write-Host "Verifica NON superata" -ForegroundColor Red
    Write-Host
    break  
}
else {
    Write-Host "Hash dello script di sorteggio: $scriptHashStringFile"
    Write-Host "Hash atteso: $scriptHashString"
    Write-Host "Verifica superata. L'hash calcolato dello script di sorteggio corrisponde a quello indicato nel file output." -ForegroundColor Blue
    Write-Host

}
# Verifica che l'hash del file xml calcolato e quello nel file output coincidano
if ($xmlHashString -ne $xmlHashStringFile) {
    Write-Host "ERRORE: hash chain compromessa! Hash del file xml diverso da quello indicato nel file output." -ForegroundColor Red
    Write-Host
    Write-Host "Hash del file xml nel file di output: $xmlHashStringFile"
    Write-Host "Hash atteso: $xmlHashString"
    Write-Host "Verifica NON superata" -ForegroundColor Red
    Write-Host
    break  
}
else {
    Write-Host "Hash del file xml: $xmlHashStringFile"
    Write-Host "Hash atteso: $xmlHashString"
    Write-Host "Verifica superata. L'hash calcolato del file xml corrisponde a quello indicato nel file output." -ForegroundColor Blue
    Write-Host

}

# Verifica che l'hash iniziale calcolato e quello nel file output coincidano
if ($initialHashString -ne $initialHashStringFile) {
    Write-Host "ERRORE: hash chain compromessa! Hash chain iniziale diverso da quello indicato nel file output." -ForegroundColor Red
    Write-Host
    Write-Host "Hash iniziale nel file di output: $initialHashStringFile"
    Write-Host "Hash atteso: $initialHashString"
    Write-Host "Verifica NON superata" -ForegroundColor Red
    Write-Host 
    break  
}
else {
    Write-Host "Hash iniziale nel file di output: $initialHashStringFile"
    Write-Host "Hash atteso: $initialHashString"
    Write-Host "Verifica superata. L'hash iniziale calcolato corrisponde a quello indicato nel file output." -ForegroundColor Blue
    Write-Host
}

# Trova tutti gli hash correnti e i risultati dei sorteggi nel file di output
$codePattern = "Codice Procedura: (.+)"
$hashPattern = "Hash corrente Commissione ([A-Za-z0-9_]+): ([A-Fa-f0-9]+)"
$resultPattern = "(?s)Composizione della Commissione a seguito del sorteggio:(.+?)Hash corrente"
$codeMatches = [regex]::Matches($outputContent, $codePattern)
$hashMatches = [regex]::Matches($outputContent, $hashPattern)
$resultMatches = [regex]::Matches($outputContent, $resultPattern)

# Verifica la correttezza dei sorteggi e l'integrità dell'hash chain
$isValid = $true
$previousHash = $initialHashString

for ($i = 0; $i -lt $hashMatches.Count; $i++) {
	$codiceProocedura = $codeMatches[$i].Groups[1].Value.Trim()
    $currentHash = $hashMatches[$i].Groups[2].Value
    $result = $resultMatches[$i].Groups[1].Value.Trim()

    # Calcola l'hash del risultato del sorteggio concatenato con l'hash precedente
    $commissioneCompleta = $result + "`n" + $previousHash
    $computedHash = $hashAlgorithm.ComputeHash([System.Text.Encoding]::UTF8.GetBytes("$result$previousHash"))
    $computedHashString = [System.BitConverter]::ToString($computedHash).Replace("-", "")
	
	    Write-Host "Composizione della Commissione della procedura $codiceProocedura, a seguito del sorteggio:" -ForegroundColor DarkGreen 
        Write-Host $result
		Write-Host
        Write-Host "Hash nel file output: $currentHash"
        Write-Host "Hash calcolato: $computedHashString"
		Write-Host "Verifica superata" -ForegroundColor Blue
        Write-Host

    # Verifica se l'hash calcolato corrisponde all'hash corrente
    if ($computedHashString -ne $currentHash) {
        $isValid = $false
        Write-Host "ERRORE: Sorteggio della procedura $codiceProocedura NON VALIDABILE" -ForegroundColor Red
        Write-Host "Composizione della Commissione della procedura $codiceProocedura, a seguito del sorteggio:" -ForegroundColor Red
        Write-Host $result
		Write-Host
        Write-Host "Hash nel file output: $currentHash"
        Write-Host "Hash calcolato: $computedHashString"
		Write-Host "Verifica NON superata" -ForegroundColor Red
        Write-Host
    }

    # Aggiorna l'hash precedente con l'hash corrente per il prossimo ciclo
    $previousHash = $currentHash
}

# Stampa il risultato della verifica
if ($isValid) {
    Write-Host "Tutti i sorteggi sono validi, hash chain integra." -ForegroundColor Green
} else {
    Write-Host "ERRORE: Alcuni sorteggi non sono validi o l'hash chain è compromessa." -ForegroundColor Red
}

# SIG # Begin signature block
# MIIJdwYJKoZIhvcNAQcCoIIJaDCCCWQCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCA7gM6JZw5ZN44a
# 2CStclX+YrmE4WNlMeUef3bEzStc9qCCBqYwggaiMIIFiqADAgECAhMUAAA4ACOW
# WHrkkDSGAAIAADgAMA0GCSqGSIb3DQEBDAUAMF4xEjAQBgoJkiaJk/IsZAEZFgJp
# dDEWMBQGCgmSJomT8ixkARkWBnBvbGltaTEZMBcGCgmSJomT8ixkARkWCXJlc291
# cmNlczEVMBMGA1UEAxMMUkVTUE9MSU1JLUNBMB4XDTIzMDcwNzA3MjcyOFoXDTI1
# MDcwNjA3MjcyOFowgYQxEjAQBgoJkiaJk/IsZAEZFgJpdDEWMBQGCgmSJomT8ixk
# ARkWBnBvbGltaTEZMBcGCgmSJomT8ixkARkWCXJlc291cmNlczEOMAwGA1UECxMF
# QVNJQ1QxFzAVBgNVBAsTDlNlcnZpY2VBY2NvdW50MRIwEAYDVQQDEwlzaWduIGNv
# ZGUwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQDiqhs4W/hIh8Aetq/1
# vpmFo/zsr3sW99pTbAfFfx9W3/TLME4qqHqq0PY1fMIera2RpaH+qVzR0D94HsUh
# 3XqnFt093o2eeNwwS00+AixM+vyaAvsM4nKq9bOGiaMieD2XN5KGVUp4qo1EP014
# lknMaDNqkewO8ip+2o5UJKav1VqY0/vUEANpqclUsrnzA/6Vo+uDEK5baejST1m8
# KVmEs7fX+JwZunr579CnjRRzd/VyUk+CGBga1AIINGRiN3wm9hMit2/vvlc2upvo
# qpUFEWGlFAu1SibRWUc0CJCXymS3pJMV3SBHbXSoTFIxt9UNugw3joWBtK2NVy8O
# eq0BAgMBAAGjggMwMIIDLDA8BgkrBgEEAYI3FQcELzAtBiUrBgEEAYI3FQitzguC
# mLINh6mfC4S3sQ2EsPFyfYKD9zWGrtU3AgFkAgEEMBMGA1UdJQQMMAoGCCsGAQUF
# BwMDMA4GA1UdDwEB/wQEAwIHgDAbBgkrBgEEAYI3FQoEDjAMMAoGCCsGAQUFBwMD
# MB0GA1UdDgQWBBShFtxtLTEqeK0E7GXgtp3orXUaXDAfBgNVHSMEGDAWgBTT9fh4
# E9CpU/OjasF4LdDRws9HuzCB2QYDVR0fBIHRMIHOMIHLoIHIoIHFhoHCbGRhcDov
# Ly9DTj1SRVNQT0xJTUktQ0EsQ049VldSRVNMSUNBMDEsQ049Q0RQLENOPVB1Ymxp
# YyUyMEtleSUyMFNlcnZpY2VzLENOPVNlcnZpY2VzLENOPUNvbmZpZ3VyYXRpb24s
# REM9cmVzb3VyY2VzLERDPXBvbGltaSxEQz1pdD9jZXJ0aWZpY2F0ZVJldm9jYXRp
# b25MaXN0P2Jhc2U/b2JqZWN0Q2xhc3M9Y1JMRGlzdHJpYnV0aW9uUG9pbnQwggEC
# BggrBgEFBQcBAQSB9TCB8jCBtgYIKwYBBQUHMAKGgalsZGFwOi8vL0NOPVJFU1BP
# TElNSS1DQSxDTj1BSUEsQ049UHVibGljJTIwS2V5JTIwU2VydmljZXMsQ049U2Vy
# dmljZXMsQ049Q29uZmlndXJhdGlvbixEQz1yZXNvdXJjZXMsREM9cG9saW1pLERD
# PWl0P2NBQ2VydGlmaWNhdGU/YmFzZT9vYmplY3RDbGFzcz1jZXJ0aWZpY2F0aW9u
# QXV0aG9yaXR5MDcGCCsGAQUFBzABhitodHRwOi8vVldSRVNMSUNBMDEucmVzb3Vy
# Y2VzLnBvbGltaS5pdC9vY3NwMDgGA1UdEQQxMC+gLQYKKwYBBAGCNxQCA6AfDB1z
# aWduLmNvZGVAcmVzb3VyY2VzLnBvbGltaS5pdDBOBgkrBgEEAYI3GQIEQTA/oD0G
# CisGAQQBgjcZAgGgLwQtUy0xLTUtMjEtMTM0NTMyNTU4MC04ODU5MDMyMC0zMTM2
# MjU0NTQ5LTEyODkwMA0GCSqGSIb3DQEBDAUAA4IBAQA5MAdrgNAAJqugGSkK7yyA
# FAAFzWlS443m+qtQFnN8mjatdCDm26xN8/srqBzwqKCNenkhJjw8ne173ppgO9Um
# /5yaWimSlsyLswrzuk9UxDjIeAh+gL3HKamZB53/kmeKpKLOkGR9sbUMV/mYKKUf
# AIRHsnMQQODcEBv2022E+Fg9MCw8LOMT+p6uB2zaXdubvkEtI1W4KtV4USHnC1zv
# U86p5eoe+FhqphaDgoK9jW7eP+dbSxM68WVe8EJ1RHyyeEXdBNoHs0ycJ0/oKYX/
# ej1ZXOJuVJuEtUlIxRBrPIBGfaOQ7rT2iIIL45tJ7sAXmK09lvpU6SIlI1ehylIp
# MYICJzCCAiMCAQEwdTBeMRIwEAYKCZImiZPyLGQBGRYCaXQxFjAUBgoJkiaJk/Is
# ZAEZFgZwb2xpbWkxGTAXBgoJkiaJk/IsZAEZFglyZXNvdXJjZXMxFTATBgNVBAMT
# DFJFU1BPTElNSS1DQQITFAAAOAAjllh65JA0hgACAAA4ADANBglghkgBZQMEAgEF
# AKCBhDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgor
# BgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMC8GCSqGSIb3
# DQEJBDEiBCBETe1pkmtCALtlMdSzzrMCOZBnWwqoKWb/6PlaXhrsJjANBgkqhkiG
# 9w0BAQEFAASCAQDI2KLh+bS1J+1aUll7FaUAqYjIIahru4S6+yOfr5Xb0ReWQ0er
# n5kFQt64KrtGqo1gl1WpdhiBV0oCf4aaJ8tE3KP8uu5WWLuDWV71RdnUn1+nqY2X
# GEMtcgSZ+z11ectY0CWNUY/KlQMIanKGkE4jWvGP5fq2c0LPzgclkgtdZ/SxWQPx
# 1b0Y8Pc8InwM2unpEmOH4eyXzX91PHxIwMHc+mwF9PkkzBGNh2Na1+iXlGC8hJsc
# PNmjBxNkNDMqS6vRyBX81jI1xcx3ZF8wYOKKu7idutwy4o0vu/duRYReYesE4XHi
# niMmD8PTcHhPtvZ1Bb1HxA7h6Vvvbqnwuamr
# SIG # End signature block
